#include <QtGui/QApplication>
#include "mainwindow.h"
#include "lyricsexternal.h"


void state(int type, QString out) {

	if(type == 1) {
		std::cout << "[INFO] " << out.toStdString() << std::endl;
	} else if(type == 2) {
		std::cout << "[ERROR] " << out.toStdString() << std::endl;
	} else if(type == 3) {
		std::cout << std::endl << ">>> [CRITICAL ERROR] <<<" << std::endl << out.toStdString() << std::endl << std::endl;
	}

}

// This function is for catching SQL errors
void SQLERR(QSqlDatabase database,QSqlQuery query, QString where) {

	if(query.lastError().text().trimmed() != "") {
		state(2,"in SQL QUERY in '" + where + "'; SQL error message: '" + query.lastError().text() + "'");
	}

	if(database.lastError().text().trimmed() != "") {
		state(2,"in DATABASE OPERATION in '" + where + "'; SQL error message: '" + database.lastError().text() + "'");
	}

}

int main(int argc, char *argv[]) {

	QApplication a(argc, argv);

	state(1,"Starting RRipper");


	QDir dir(QDir::homePath() + "/.RRipper/");
	if(!dir.exists()) {
		dir.mkdir(QDir::homePath() + "/.RRipper");
	}

	QFile global_pref_file(QDir::homePath() + "/.RRipper/preferences");
	if(!global_pref_file.exists()) {
		if(global_pref_file.open(QIODevice::ReadWrite)) {
			state(1,"Setting standard global preferences");

			QTextStream out(&global_pref_file);
			QString content = "NotMatching=0\n"
					  + QString("DefaultLocation=") + QDir::homePath() + "/\n"
					  + "MPlayer=/usr/bin/mplayer\n"
					  + "Streamripper=/usr/bin/streamripper\n"
					  + "Language=0\n"
					  + "LyricsAutoFetch=1";
			out << content;
			global_pref_file.close();
		}
	}


	// Possibly initialise the translator (for german)
	QTranslator qtTranslator;
	qtTranslator.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
	a.installTranslator(&qtTranslator);

	QTranslator myappTranslator;

	QFile file_trans(QDir::homePath() + "/.RRipper/preferences");
	if(file_trans.open(QIODevice::ReadOnly)) {

		QTextStream in(&file_trans);
		QString all = in.readAll();

		all = all.split("Language=").at(1).split("\n").at(0);

		if(all == "1") {

			myappTranslator.load(":/rripper_de.qm");
			a.installTranslator(&myappTranslator);

			state(1,"Lade deutsche Übersetzung");

		}

	}


	// Load the file ~/.RRipper/.locked
	QFile file(QDir::homePath() + "/.RRipper/.locked");
	if(!file.open(QIODevice::ReadWrite)) {
		state(3,"The read/write acces to '" + QDir::homePath() + "/.RRipper/' was rejected! Please make sure you own the access rights to this directory and try again.");
	} else {

		QTextStream timestamp(&file);

		bool ok;
		QString lastUpdate = timestamp.readAll().trimmed();

		file.close();

		QFile file_new(QDir::homePath() + "/.RRipper/.locked");
		file_new.open(QIODevice::ReadWrite);
		QTextStream timestamp_new(&file_new);

		int current = QDateTime::currentDateTime().toTime_t();
		int old = lastUpdate.toInt(&ok,10);

		// If the .locked file was 2 or les seconds before updated -> instance of RRipper currently running
		if(current - old <= 1) {

			int invalid = 0;

			for(int i = 1; i < argc; ++i) {
				if(QString(argv[i]) != "--show" && QString(argv[i]) != "--stop" && QString(argv[i]) != "--play") {
					invalid = 1;
				}
			}

			if(invalid == 1) {
				state(2,"Invalid Command entered!\n\n\tPossible arguments are:\n\t--show\t Show the RRipper Mainwindow\n\t--play\t Toggle Playing\n\n");
				state(1,"Exiting...");
			} else {

				QFile dothat(QDir::homePath() + "/.RRipper/dothat");
				if(dothat.open(QIODevice::ReadWrite)) {
					QTextStream out(&dothat);

					int atleastone = 0;

					for(int i = 1; i < argc; ++i) {
						out << argv[i] << "\n";
						atleastone = 1;

						if(QString(argv[i]) == "--show") {
							state(1,"Showing RRipper Mainwindow");
						} else if(QString(argv[i]) == "--play") {
							state(1,"Toggle Playing of RRipper");
						}
					}

					if(atleastone == 0) {
						out << "--show\n";
						state(1,"Running RRipper instance found");
						state(1,"Showing RRipper Mainwindow");
					}

					state(1,"End.");

				}

				dothat.close();

			}

		} else {

			QSqlDatabase directories_test = QSqlDatabase::addDatabase("QSQLITE","directories_test");
			directories_test.setDatabaseName(QDir::homePath() + "/.RRipper/directories.db");
			directories_test.setHostName("localhost");
			directories_test.setUserName("rripper");
			directories_test.setPassword("");

			QSqlQuery query_directories(directories_test);

			// Open the directories database and make sure all columns are there
			if(directories_test.open("rripper","")) {

				query_directories.clear();

				query_directories.exec("CREATE TABLE IF NOT EXISTS shoutcast_stations (description TEXT)");
				SQLERR(directories_test,query_directories,"Starting (Create shoutcast_stations)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN name TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD name)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN description TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD description)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN mime TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD mime)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN id TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD id)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN bitrate TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD bitrate)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN genre_searched TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD genre_searched)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_stations ADD COLUMN genre_all TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_stations ADD genre_all)");
				query_directories.clear();


				query_directories.exec("CREATE TABLE IF NOT EXISTS shoutcast_genres (genre TEXT)");
				SQLERR(directories_test,query_directories,"Starting");
				query_directories.clear();

				query_directories.exec("ALTER TABLE shoutcast_genres ADD COLUMN genre TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (shoutcast_genres ADD genre)");
				query_directories.clear();


				query_directories.exec("CREATE TABLE IF NOT EXISTS icecast_stations (name TEXT)");
				SQLERR(directories_test,query_directories,"Starting");
				query_directories.clear();

				query_directories.exec("ALTER TABLE icecast_stations ADD COLUMN name TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (icecast_stations ADD name)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE icecast_stations ADD COLUMN url TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (icecast_stations ADD url)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE icecast_stations ADD COLUMN mime TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (icecast_stations ADD mime)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE icecast_stations ADD COLUMN bitrate TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (icecast_stations ADD bitrate)");
				query_directories.clear();

				query_directories.exec("ALTER TABLE icecast_stations ADD COLUMN genre TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (icecast_stations ADD genre)");
				query_directories.clear();


				query_directories.exec("CREATE TABLE IF NOT EXISTS icecast_genres (genre TEXT)");
				SQLERR(directories_test,query_directories,"Starting");
				query_directories.clear();

				query_directories.exec("ALTER TABLE icecast_genres ADD COLUMN genre TEXT");
				if(!query_directories.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_directories,"Starting (icecast_genres ADD genre)");
				query_directories.clear();

			}

			directories_test.close();

			SQLERR(directories_test,query_directories,"Starting (Close directories)");


			QSqlDatabase stations_test = QSqlDatabase::addDatabase("QSQLITE","stations_test");
			stations_test.setDatabaseName(QDir::homePath() + "/.RRipper/stations.db");
			stations_test.setHostName("localhost");
			stations_test.setUserName("rripper");
			stations_test.setPassword("");

			QSqlQuery query_stations(stations_test);

			// Do the same with the database for the stations
			if(stations_test.open()) {

				query_stations.clear();

				query_stations.exec("CREATE TABLE IF NOT EXISTS details (code TEXT)");
				SQLERR(directories_test,query_stations,"Starting (Create details)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN D TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD D)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN s TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD s)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN r TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD r)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN a_checked TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD a_checked)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN a_text TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD a_text)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN o TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD o)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN c TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD c)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN l TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD l)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN M TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD M)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN q TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD q)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN u TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD u)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN i TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD i)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN wid3v1 TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD wid3v1)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN woid3v2 TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD woid3v2)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN name TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD name)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN url TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD url)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN location TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD location)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN server TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD server)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN bitrate TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD bitrate)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN metainterval TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD metainterval)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN codec TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD codec)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN swapartisttitle TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD swapartisttitle)");
				query_stations.clear();

				query_stations.exec("ALTER TABLE details ADD COLUMN playback TEXT");
				if(!query_stations.lastError().text().startsWith("duplicate column name"))
					SQLERR(directories_test,query_stations,"Starting (details ADD playback)");
				query_stations.clear();

			}


			QFile file(QDir::homePath() + "/.RRipper/streams");

			QSqlQuery query(stations_test);
			query.exec("SELECT COUNT (code) FROM details");
			query.next();

			QString amount = query.value(0).toString();

			query.clear();

			int StreamFileImported = 0;

			// If with previous RRipper versions a stream file was created -> 'translate' into database
			if(amount == "0" && file.exists(QDir::homePath() + "/.RRipper/streams")) {

				if(file.open(QIODevice::ReadOnly)) {

					state(1,"Importing stream file used by previous versions of RRipper.");

					QTextStream old_list(&file);

					QStringList content = old_list.readAll().split("\n");

					stations_test.transaction();

					QSqlQuery query_values(stations_test);
					query_values.prepare("INSERT INTO details (code,D,s,r,a_checked,a_text,o,c,l,M,q,u,i,wid3v1,woid3v2,name,url,location,server,bitrate,metainterval,codec,swapartisttitle,playback) VALUES (:code,:D,:s,:r,:a_checked,:a_text,:o,:c,:l,:M,:q,:u,:i,:wid3v1,:woid3v2,:name,:url,:location,:server,:bitrate,:metainterval,:codec,:swapartisttitle,:playback)");

					SQLERR(stations_test,query_values,"Prepare");

					int pref_amount = 16;

					int stations = 0;
					int url = 0;
					int location = 0;
					int D = 0;
					int s = 0;
					int r = 0;
					int a = 0;
					int o = 0;
					int c = 0;
					int l = 0;
					int M = 0;
					int q = 0;
					int u = 0;
					int sr_i = 0;
					int wid3v1 = 0;
					int woid3v2 = 0;

					for(int i = 0; i < content.size(); ++i) {

						QStringList temp;
						temp.clear();

						if(content.at(i) == "") {
							temp  << "-=";
						} else {
							temp = content.at(i).split("=");
						}

						if(temp.at(0).startsWith("station")) {

							if(temp.size() == 1) {
								query_values.bindValue(":name","[Unknown]");
							} else if(temp.size() == 2) {
								query_values.bindValue(":name",temp.at(1));
							} else if(temp.size() > 2) {
								temp.removeFirst();
								query_values.bindValue(":name",temp.join("="));
							}

							stations = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"name");

						} else if(temp.at(0).startsWith("url")) {

							if(temp.size() == 1) {
								query_values.bindValue(":url","http://");
							} else if(temp.size() == 2) {
								query_values.bindValue(":url",temp.at(1));
							} else if(temp.size() > 2) {
								temp.removeFirst();
								query_values.bindValue(":url",temp.join("="));
							}

							url = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"url");

						} else if(temp.at(0).startsWith("location")) {

							if(temp.size() == 1) {
								query_values.bindValue(":location",QDir::homePath() + "/");
							} else if(temp.size() == 2) {
								QString temp_dir = temp.at(1);
								if(!temp_dir.endsWith("/")) {
									temp_dir = temp_dir + "/";
								}
								query_values.bindValue(":location",temp_dir);
							} else if(temp.size() > 2) {
								temp.removeFirst();
								QString temp_joined = temp.join("=");
								if(!temp_joined.endsWith("/")) {
									temp_joined = temp_joined + "/";
								}
								query_values.bindValue(":location",temp_joined);
							}

							location = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"location");

						} else if(temp.at(0).startsWith("D")) {

							if(temp.size() == 1){
								query_values.bindValue(":D","");
							} else if(temp.size() == 2) {
								QRegExpValidator *valid = new QRegExpValidator(QRegExp("^([%][ATDdSq]){1}(/[%][ATDdSq]){0,}$"),0);
								int pos = 0;
								QString string = temp.at(1);
								if(valid->validate(string,pos)) {
									query_values.bindValue(":D",temp.at(1));
								} else {
									query_values.bindValue(":D","%A/%T");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":D","%A/%T");
							}

							D = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"D");

						} else if(temp.at(0).startsWith("s")) {

							if(temp.size() == 1) {
								query_values.bindValue(":s","0");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "0" || temp.at(1) == "1") {
									query_values.bindValue(":s",temp.at(1));
								} else {
									query_values.bindValue(":s","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":s","0");
							}

							s = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"s");

						} else if(temp.at(0).startsWith("r")) {

							if(temp.size() == 1) {
								query_values.bindValue(":r","0");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "0" || temp.at(1) == "1") {
									query_values.bindValue(":r",temp.at(1));
								} else {
									query_values.bindValue(":r","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":r","0");
							}

							r = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"r");

						} else if(temp.at(0).startsWith("a")) {

							if(temp.size() == 1) {
								query_values.bindValue(":a_checked","1");
								query_values.bindValue(":a_text","");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "-----") {
									query_values.bindValue(":a_checked","0");
									query_values.bindValue(":a_text","");
								} else {
									QRegExpValidator *valid = new QRegExpValidator(QRegExp("^([%][ATDdSq]){1}(/[%][ATDdSq]){0,}$"), 0);
									int pos = 0;
									QString string = temp.at(1);
									if(valid->validate(string,pos)) {
										query_values.bindValue(":a_checked","1");
										query_values.bindValue(":a_text",temp.at(1));
									} else {
										query_values.bindValue(":a_checked","0");
										query_values.bindValue(":a_text","");
									}
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":a_checked","0");
								query_values.bindValue(":a_text","");
							}

							a = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"a");

						} else if(temp.at(0).startsWith("o")) {

							if(temp.size() == 1) {
								query_values.bindValue(":o","-1");
							} else if(temp.size() == 2) {
								bool ok;
								int o = temp.at(1).toInt(&ok,10);
								if(o > -1 && o < 4 && ok == true) {
									query_values.bindValue(":o",temp.at(1));
								} else {
									query_values.bindValue(":o","-1");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":o","-1");
							}

							o = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"o");

						} else if(temp.at(0).startsWith("c")) {

							if(temp.size() == 1) {
								query_values.bindValue(":c","0");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "0" || temp.at(1) == "1") {
									query_values.bindValue(":c",temp.at(1));
								} else {
									query_values.bindValue(":c","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":c","0");
							}

							c = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"c");

						} else if(temp.at(0).startsWith("l")) {

							if(temp.size() == 1) {
								query_values.bindValue(":l","0");
							} else if(temp.size() == 2) {
								bool ok;
								int l = temp.at(1).toInt(&ok,10);
								if(l > -1 && l < 301 && ok == true) {
									query_values.bindValue(":l",temp.at(1));
								} else {
									query_values.bindValue(":l","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":l","0");
							}

							l = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"l");

						} else if(temp.at(0).startsWith("M")) {

							if(temp.size() == 1) {
								query_values.bindValue(":M","0");
							} else if(temp.size() == 2) {
								bool ok;
								int M = temp.at(1).toInt(&ok,10);
								if(M > -1 && M < 1000 && ok == true) {
									query_values.bindValue(":M",temp.at(1));
								} else {
									query_values.bindValue(":M","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":M","0");
							}

							M = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"M");

						} else if(temp.at(0).startsWith("q")) {

							if(temp.size() == 1) {
								query_values.bindValue(":q","0");
							} else if(temp.size() == 2) {
								bool ok;
								int q = temp.at(1).toInt(&ok,10);
								if(q > -1 && q < 10000 && ok == true) {
									query_values.bindValue(":q",temp.at(1));
								} else {
									query_values.bindValue(":q","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":q","0");
							}

							q = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"q");

						} else if(temp.at(0).startsWith("u")) {

							if(temp.size() == 1) {
								query_values.bindValue(":u","");
							} else if(temp.size() == 2) {
								query_values.bindValue(":u",temp.at(1));
							} else if(temp.size() > 2) {
								temp.removeFirst();
								query_values.bindValue(":u",temp.join("="));
							}

							u = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"u");

						} else if(temp.at(0).startsWith("i")) {

							if(temp.size() == 1) {
								query_values.bindValue(":i","0");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "0" || temp.at(1) == "1") {
									query_values.bindValue(":i",temp.at(1));
								} else {
									query_values.bindValue(":i","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":i","0");
							}

							sr_i = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"i");

						} else if(temp.at(0).startsWith("wid3v1")) {

							if(temp.size() == 1) {
								query_values.bindValue(":wid3v1","0");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "0" || temp.at(1) == "1") {
									query_values.bindValue(":wid3v1",temp.at(1));
								} else {
									query_values.bindValue(":wid3v1","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":wid3v1","0");
							}

							wid3v1 = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"wid3v1");

						} else if(temp.at(0).startsWith("woid3v2")) {

							if(temp.size() == 1) {
								query_values.bindValue(":woid3v2","0");
							} else if(temp.size() == 2) {
								if(temp.at(1) == "0" || temp.at(1) == "1") {
									query_values.bindValue(":woid3v2",temp.at(1));
								} else {
									query_values.bindValue(":woid3v2","0");
								}
							} else if(temp.size() > 2) {
								query_values.bindValue(":woid3v2","0");
							}

							woid3v2 = 1;
							--pref_amount;

							SQLERR(stations_test,query_values,"woid3v2");

						}

						if(pref_amount == 0 || (temp.at(0).startsWith("station") && pref_amount < 15)) {

							if(stations == 0) {
								query_values.bindValue(":name","[Unknown]");
							}
							if(url == 0) {
								query_values.bindValue(":url","http://");
							}
							if(location == 0) {
								query_values.bindValue(":location",QDir::homePath());
							}
							if(D == 0) {
								query_values.bindValue(":D","%A/%T");
							}
							if(s == 0) {
								query_values.bindValue(":s","0");
							}
							if(r == 0) {
								query_values.bindValue(":r","1");
							}
							if(a == 0) {
								query_values.bindValue(":a_checked","0");
								query_values.bindValue(":a_text","");
							}
							if(o == 0) {
								query_values.bindValue(":o","2");
							}
							if(c == 0) {
								query_values.bindValue(":c","0");
							}
							if(l == 0) {
								query_values.bindValue(":l","0");
							}
							if(M == 0) {
								query_values.bindValue(":M","0");
							}
							if(q == 0) {
								query_values.bindValue(":q","0");
							}
							if(u == 0) {
								query_values.bindValue(":u","");
							}
							if(sr_i == 0) {
								query_values.bindValue(":i","0");
							}
							if(wid3v1 == 0) {
								query_values.bindValue(":wid3v1","0");
							}
							if(woid3v2 == 0) {
								query_values.bindValue(":woid3v2","0");
							}

							SQLERR(stations_test,query_values,"rest");


							// Get a new code
							QStringList possibleCharacters;
							possibleCharacters << "a" << "b" << "c" << "d" << "e" << "f" << "g" << "h" << "i" << "j" << "k" << "l" << "m" << "n" << "o" << "p" << "q" << "r" << "s" << "t" << "u" << "v" << "w" << "x" << "y" << "z"
									<< "1" << "2" << "3" << "4" << "5" << "6" << "7" << "7" << "8" << "9" << "0"
									<< "#" << "+" << "-" << "!" << "=" << "?" << "&" << "$";

							QString code = "";

							int new_string = 0;

							while(new_string == 0) {

								for(int i = 0; i < 10; ++i) {

									int random = qrand() % possibleCharacters.size();

									code += possibleCharacters.at(random);

								}

								QSqlQuery query_code(stations_test);

								query_code.exec("SELECT COUNT(code) FROM details WHERE code LIKE '" + code + "'");
								query_code.next();
								if(query_code.value(0).toString() == "0") {
									new_string = 1;
								}
								query_code.clear();

							}

							query_values.bindValue(":code",code);
							query_values.bindValue(":playback","normal");
							query_values.bindValue(":swapartisttitle","0");
							query_values.bindValue(":codec","");
							query_values.bindValue(":server","---");
							query_values.bindValue(":bitrate","---");
							query_values.bindValue(":metainterval","---");

							SQLERR(stations_test,query_values,"new");

							query_values.exec();
							pref_amount = 16;

							StreamFileImported = 1;

							SQLERR(stations_test,query_values,"exec");

							stations = 0;
							url = 0;
							location = 0;
							D = 0;
							s = 0;
							r = 0;
							a = 0;
							o = 0;
							c = 0;
							l = 0;
							M = 0;
							q = 0;
							u = 0;
							sr_i = 0;
							wid3v1 = 0;
							woid3v2 = 0;

						}

					}

					stations_test.commit();

					SQLERR(stations_test,query_values,"Commit");

				}

			// If no stream file exists create standard
			} else if(amount == "0" && !file.exists(QDir::homePath() + "/.RRipper/streams")) {

				state(1,"Setting default station entry");

				QSqlQuery query_values(stations_test);

				query_values.exec("INSERT INTO details (codec,playback,swapartisttitle,a_text,code,D,s,r,a_checked,o,c,l,M,q,u,i,wid3v1,woid3v2,name,url,location,server,bitrate,metainterval) VALUES ('','normal','0','','GIRFTDAHRF','%A/%T','0','1','0','2','0','0','0','0','','0','0','0','[New Stream]','http://','" + QDir::homePath() + "/','---','---','---')");

				SQLERR(stations_test,query_values,"CreateFirstEntry");

				query_values.clear();

			}


			stations_test.close();

			SQLERR(stations_test,query_stations,"Close stations");


			timestamp_new << QDateTime::currentDateTime().toTime_t();

			MainWindow w;
			w.show();
			if(StreamFileImported == 1) {
				QMessageBox::information(0,"Info","Your stream file used by previous RRipper versions has been imported. I gave my best, but nevertheless, please control the preferences first.");
			}
			return a.exec();

		}

		file_new.close();

	}

	file.close();

}

