#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <iostream>
#include <string>
#include <QMainWindow>
#include <QMap>
#include <QMapIterator>
#include <QDir>
#include <QMessageBox>
#include <QTextStream>
#include <QProcess>
#include <QFileDialog>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QDomDocument>
#include <QTreeWidgetItem>
#include <QCloseEvent>
#include <QLibraryInfo>
#include <QTranslator>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QPalette>
#include <QResizeEvent>
#include <QTimer>
#include <QTime>
#include <QSystemTrayIcon>
#include <QTextDocument>

namespace Ui {
	class MainWindow;
}

class Lyrics;

class MainWindow : public QMainWindow {

	Q_OBJECT

public:
	Lyrics *external;
	void enableButton();
	void enableButton_exe();

	int lyricsSource;
	int musicsonglyricsTRY;

	MainWindow(QWidget *parent = 0);
	~MainWindow();

	QString version;					// This string contains the version (e.g.: 'RRipper 0.9.2')
	QMap<QString,QProcess*> stations_processes;		// This QMap contains the streamripper processes (can't be saved in a database)

	QTimer *lock;						// When an instance of RRipper is running, a file is updated every 2 seconds to show, that it is locked

	QSqlDatabase stations;					// Connection to the database containing the station data
	QSqlDatabase directories;

	// The Network Access Manager to get the directories data
	QNetworkAccessManager *requestshoutcastgenres;
	QNetworkAccessManager *requestshoutcaststations;
	QNetworkAccessManager *requesticecastdata;

	// And the NetworkAccessManager to get the lyrics
	QNetworkAccessManager *requestlyrics;

	QString genrecurrentlyloading;				// This string contains the genre, that is currently loading
	QStringList expanded;					// This stringlist contains all the genres, that are expanded
	int IComeFromDisplayStations;				// The function DisplayStations() expand genres and
	int CurrentCollapsingOutOfFunction;			// The function DirectoryAllCollapse() uses an iterator, so we can't change the value of the StringList during the function
	QTreeWidgetItem *itemJustExpanded;			// This TreeWidgetItem contains the last expanded item

	QString codeOfLastAdded;				// This string contains the code of the last added item (set by AddNewStation(), necessary for AddStationToList())

	QProcess *mplayer;					// The mplayer process for playing
	QProcess *streamripperforplaying;			// The streamripper process for the intelligent playback
	QString playinglocalhost;				// This string contains the relay host for mplayer (used by playing a recording and for the intelligent playback)
	QString played;						// Here the code of the currently played station is saved
	int playingstarted;					// When a station was played and 'StartRecording' was clicked, the recording is restarted as soon as the connection of sttreamripper is there
	int torestart;						// Does a station have to be restarted as soon as possible?
	QString playingFileID3;					// Needed by the intelligent playback. Contains the filename of the currently ripped song in the temporary folder

	QMap<QString,QMap<QString,QString> > values;		// This QMap contains some temporary information about the stations (such as state, etc.)
	QMap<QString,QString> played_values;			// The values received by mplayer (such as server, etc.). Used when no information of a previous streamripper process is available

	QStringList CreateStreamripperPrefString(QString code);	// This stringlist function creates and returns a stringlist containing the streamripper preferences of the station with the code 'code'


	QAction *closeWindow;

	// This are the QActions for the menu and the toolbar. Created manually, because the designer has some problems in setting up submenus in the menubar
	QAction *sel_delete;
	QAction *sel_record;
	QAction *sel_play;
	QAction *sel_stopAfterSong;

	QAction *all_record;
	QAction *all_stop;
	QAction *all_stopAfterSong;

	QSlider *volume;
	QLabel *volume_value;
	QPushButton *volumeMute;
	int volumeBeforeMuting;
	QPushButton *save_song;

	// These are some global values, set up globally to simplify the translation process
	QString id3_artist;
	QString id3_title;
	QString status_notconnected;
	QString status_connecting;
	QString status_receivingdetails;
	QString status_gettingtrackname;
	QString status_buffering;
	QString status_skipping;
	QString status_ripping;
	QString status_playing;
	// The following two strings are containing the standard error text for an error message
	QString ERR;
	QString ERRT;

	// These values are used by the function ResizeColumns() to set up the column size of the list widget automatically
	double rowone;
	double rowtwo;
	double rowthree;
	double rowfour;
	double rowfive;
	double rowsix;

	// A function to simplify the catching of SQL errors
	void SQLERR(QSqlDatabase database,QSqlQuery query,QString where);

	// A function to delete a directory recursively (used for the temporary folder /tmp/rripper/, run every start of an intelligent playback
	void DeleteDirectoryRecursive(QString directory);

	// Every time the window is resized, this function is called to resize the column width of the list widget
	void ResizeColumns();

	// Here the global preferences are loaded
	void LoadGlobalPreferences();

	int windowMaximised;
	int quitOrCloseRRipper;

	void UpdateLyricsInterfaces(QString source, QString lyrics, int enabled);

	// Printing the [INFO]/[ERROR] Messages as console output
	void state(int type,QString out);

public slots:

	void showLyricsExternal();

	void locked();				// Every two seconds this function is called to ensure, that only one RRipper instance is running

	void SelectEntry(int a, int b);		// When a new entry in the list is selected, load the preferences
	void DisEnablePrefDetailsFields();	// Some Streamripper preferences need additional infos -> dis-/enable fields according to checked status of checkbox
	void ValidateStreamripperPrefs();	// Some streamripper preferences don't go together -> check
	void RegExpPrefFields();		// The streamripper preferences '-D' and '-s' have to have a specific pattern
	void UpdateStationsDatabase();		// When some of the permanent information is changed -> save changes to database
	void SaveGlobalPreferences();		// When the global preferences are changed -> save

	void SelectDefaultLocation();		// Select the default saving location for a new station
	void SelectMPlayerLocation();		// Change the executeable MPlayer file
	void SelectStreamripperLocation();	// Change the executeable Streamripper file

	// The following function are connected with the menu-/toolbar

	void QuitProgram();

	void AddNewStation();			// Add a new station to the list
	void DeleteStation();			// Delete a station from the list

	void Sel_StartRecording();		// Start recording of selected stream
	void All_StartRecording();		// Start recording of all streams
	void All_StopRecording();		// Stop recording of all streams
	void StartStopRecording();		// The above function just set a mark to start/stop (a) recording. This function finally is doing the steps to start/stop them

	void Sel_StopAfterSong();		// Stop the recording of the current stream after song
	void All_StopAfterSong();		// Stop the recording of all streams after current song

	void StartStopPlaying();		// Start/Stop the playing of selected stream

	void GetStreamripperOutput();		// Get the Streamripper output and analyse it
	void GetMplayerOutput();		// Get the MPlayer output and analyse it
	void GetStreamripperForPlayingOutput();	// Get the Streamripper output while intelligent playback and analyse it

	void ResetStationListSorting();		// Remove the sorting of the list view

	void SelectSavingLocation();		// Change the saving location fo currently selected station

	void DirectoryChanged(QString directory);	// The station directory changed (e.g. Shoutcast, Icecast, ...)

	void RequestShoutcastGenres();				// Request the Shoutcast Genres
	void AnalyseShoutcastGenres(QNetworkReply *reply);	// Analyse the Shoutcast Genres
	void AnalyseShoutcastStations(QNetworkReply *reply);	// Analyse the Shoutcast Stations

	void RequestIcecastData();				// Request the Icecast data (genres+stations)
	void AnalyseIcecastData(QNetworkReply *reply);		// Analyse the Icecast data

	void DisplayStations();					// Completely refresh the directory view
	void DirectoriesItemExpanded(QTreeWidgetItem *item);	// When an item is expanded
	void DirectoriesItemCollapsed(QTreeWidgetItem *item);	// When an item is collapsed
	void AddStationToList(QTreeWidgetItem *item);		// Add an station of the directory list to the saved list
	void DirectoryAllCollapse();				// Collapse all directory entries

	void FirstLyricsSourceChanged();
	void RequestLyrics();				// Request the lyrics of the currently played song
	void AnalyseLyrics(QNetworkReply *reply);	// Analyse and display the lyrics
	void ResetLyricFields();			// Reset the lyrics fields
	void LyricAutoFetch();				// Auto-Fetch the lyrics

	void KillAllInternalProcesses();		// Kill all the internal processes
	void KillAllExternalProcesses();		// Kill all the external processes

	void AboutRRipper();				// Display a short note about RRipper

	void showHideMainwindow();
	void showHideTrayIcon();

	void volumeChanged();
	void volumeMuted();

private:
	void LoadStationData();				// Load the station data from the database and enter them into the list view
	void CreateToolbar();				// Create the menu- and toolbar

	void createTrayIcon();
	QSystemTrayIcon *trayIcon;
	QMenu *trayIconMenu;

	QAction *toggleMainwindow;
	QAction *quitProgram;
	QAction *about;

	void showMessage(int type, QString title, QString body);


private slots:
	void iconActivated(QSystemTrayIcon::ActivationReason reason);

protected:
	void closeEvent(QCloseEvent *event);		// Close the window
	void changeEvent(QEvent *e);			// Default RRipper set-up. I'm not quite sure, why it's there, but I think it has areason ;-)
	void resizeEvent(QResizeEvent *event);		// When the window is resized -> adjust the columns of the list view

private:
	Ui::MainWindow *ui;
};

#endif // MAINWINDOW_H

